@@ -59,7 +59,15 @@ namespace CEC
     virtual void Close(void) = 0;
 
     /*!
-     * @brief Try to find all connected CEC adapters. Only implemented on Linux at the moment.
+     * @brief Set and enable the callback methods. If this method is not called, the GetNext...() methods will have to be used.
+     * @param cbParam Parameter to pass to callback methods.
+     * @param callbacks The callbacks to set.
+     * @return True when enabled, false otherwise.
+     */
+    virtual bool EnableCallbacks(void *cbParam, ICECCallbacks *callbacks) = 0;
+
+    /*!
+     * @brief Try to find all connected CEC adapters. Only implemented on Linux and Windows at the moment.
      * @param deviceList The vector to store device descriptors in.
      * @param iBufSize The size of the deviceList buffer.
      * @param strDevicePath Optional device path. Only adds device descriptors that match the given device path.
@@ -286,24 +294,24 @@ namespace CEC
 
     /*!
      * @brief Sends a volume up keypress to an audiosystem if it's present.
-     * @param bWait Wait for the response of the audiosystem when true.
+     * @param bSendRelease Send a key release after the keypress.
      * @return The new audio status.
      */
-    virtual uint8_t VolumeUp(bool bWait = true) = 0;
+    virtual uint8_t VolumeUp(bool bSendRelease = true) = 0;
 
     /*!
      * @brief Sends a volume down keypress to an audiosystem if it's present.
-     * @param bWait Wait for the response of the audiosystem when true.
+     * @param bSendRelease Send a key release after the keypress.
      * @return The new audio status.
      */
-    virtual uint8_t VolumeDown(bool bWait = true) = 0;
+    virtual uint8_t VolumeDown(bool bSendRelease = true) = 0;
 
     /*!
      * @brief Sends a mute keypress to an audiosystem if it's present.
-     * @param bWait Wait for the response of the audiosystem when true.
+     * @param bSendRelease Send a key release after the keypress.
      * @return The new audio status.
      */
-    virtual uint8_t MuteAudio(bool bWait = true) = 0;
+    virtual uint8_t MuteAudio(bool bSendRelease = true) = 0;
 
     /*!
      * @brief Send a keypress to a device on the CEC bus.
@@ -352,7 +360,6 @@ namespace CEC
     virtual const char *ToString(const cec_system_audio_status mode) = 0;
     virtual const char *ToString(const cec_audio_status status) = 0;
     virtual const char *ToString(const cec_vendor_id vendor) = 0;
-
   };
 };
 
